;==========================================================================================
; Programm zur Steuerung des MCP2515 Bausteins (CAN-BUS) mit SX52 Microcontroller
;
; Visitec GmbH Januar 2008 by N.Fritsch   	     
; 
; MCP2515_2008.inc		(Speicherbereich I  $A00 - $BF1)
; 				(Speicherbereich II $C00 - $C4B)
;==========================================================================================
;
; DEFINIERTE ROUTINEN SIND:
;
;	- Definitionen von Konstanten fr MCP2515
;	- Makros zur Initialisierung und zur
;	- Kommunikation mit der SPI - Schnittstelle des MCP2515
;
;
;	Beschreibung der enthaltenen Hilfsmakros
;	========================================
;
;	MCP2515_Select		-	MCP2515 anwhlen
;	MCP2515_UnSelect	-	MCP2515 abwhlen
;
;	CLK_Hi			-	Clock - Ausgang des SX an den MCP auf 1
;	CLK_Lo			-	Clock auf 0
;	
;	SPI_Write		-	Macro (2 Parameter) setzt:
;						 	MCP2515_AdressReg	  (<- Para. 1)	- Adresse im MCP die gelesen werden soll
;							MCP2515_Data0 		  (<- Para. 2)	- zu schreibende Daten
;
;	SPI_Read		-	Macro (2 Parameter) setzt:
;						 	MCP2515_AdressReg	  (<- Para. 1)	- Adresse im MCP die gelesen werden soll
;					
;	SPI_BitMod		-	Macro (3 Parameter) setzt:
;						 	MCP2515_AdressReg 	  (<- Para. 1)	- Adresse im MCP die gelesen werden soll
;						 	MCP2515_MaskReg	  	  (<- Para. 2)	- diese Bits werden gendert
;							MCP2515_Data0 		  (<- Para. 3)	- zu schreibende Daten
;	
;	MCP2515 SPI INTERFACE	
;	=====================			
;	(eine sehr gute Beschreibung dieser Befehle befindet sich im Datenblatt zum MCP2515 auf den Seiten 55 - 58)
;
;	READ_INSTRUCTION			-	liest einen Register des MCP2515 aus (sequentiell)
;						-	bentigt werden:	MCP2515_AdressReg		- Adresse im MCP die gelesen werden soll
;										MCP2515_Data0			- hier wird in den SX reingeschrieben
;
;	WRITE_INSTRUCTION			-	schreibt in ein oder mehrere Register (sequentiell) des MCP2515
;						-	bentigt werden:	MCP2515_AdressReg		- Adresse im MCP die beschrieben werden soll
;										MCP2515_Data0	- das hier wird in den MCP reingeschrieben
;										MCP2515_ByteCounter		- soviel Bytes werden geschrieben
;
;
;	BIT_MODIFY_INSTRUCTION			-	im Datenblatt des MCP2515 auf S. 53 steht welche Register hiermit bearbeitet werden knnen
;						-	bentigt werden:	MCP2515_AdressReg		- Adresse im MCP die beschrieben werden soll
;										MCP2515_MaskReg		- diese Bits werden gendert
;										MCP2515_Data0	- ... und hier steht wie	
;
;	LOAD_TX_BUFFER				-	Ldt den TX Buffer mit der neu zu sendenden Can Msg
;
;	READ_RX_BUFFER				-	liest die Werte aus dem RX Buffer aus (empfangene Can Msg)
;
;	READ_ST_INSTR				-	fhrt Read Status Instruction aus
;
;	RX_ST_INSTR				-	RX Status Instruction wird ausgelesen
;
;
;	HAUPTROUTINEN FR USER
;	======================
;
;	Init_2515				-	fhrt die Initialisierung des MCP2515/MCP2515 aus
;
;	SendCanMsg				-	Makro legt die 11 Parameter in der entsprechenden Registerbank zum senden einer MSG ab
;						-	ruft anschlieend die Routine sendMSG aud
;	
;	CheckCANMsg				-	berprft ob neue Nachricht im MCP2515/MCP2515 vorhanden	
;
;	InitCANSADR				-	Setzen der Adresse fr zu Sendende Nachrichten
;
;------------------------------------------------------------------------------------------------------------------------------------------------------
;
; Verwendung im Hauptprogramm: 	include "MCP2515.inc"
;
; Bemerkungen:			Die hier definierten Subroutinen sind im Programmspeicher ab page $A00 abgelegt.
;				Zum Zugriff auf diese Routinen ist nach dem Aufruf im Hauptprogramm wieder die entsprechende Adresse zu setzen.
;				Da die ersten 3 Bits (7,6,5) des Statusregisters auf einen anderen Speicherbereich zeigen muss das 
;				Statusregisters wieder angepasst werden in diesem Fall Page0 ==> page$000
;
;				Bsp.: 	page $A00
;				        	call Init_2515
;				      	page $000
;
;				      	...
;
;					page $C00
;						call CheckCANMsg	
;					page $000
;
;				      	...
;
;				      	page $C00
;						SendCanMsg
;				      	page $000	
;
;				      	...
;
; ACHTUNG!!!:			nderungen in diesem Modul sollten nur von einem erfahrenen Pogrammierer vorgenommen werden, da
;				sich Soubroutinen verschieben knnen und somit undefinierte Sprnge im Pogrammablauf entstehen.
;				Sollten dennoch nderungen notwendig sein, muss im Debug Mode geprft werden, ob alle Sprnge 
;				dahin gehen wo sie sollen!!!
;				Zudem ist die Assemblerliste unter dem Men "Run" als Hilfe heranzuziehen.
;
;				Ein weiterer zu beachtender Punkt ist, das in den Hauptroutinen fr den USER
;				(Init_2515, SendCanMsg, CheckCANMsg) die Page am Ende der Routinen auf das Hauptpogramm ausgerichtet wird.
;				Eingestellt ist dabei page $100 (==page $000). Befindet sich das Hauptpogramm an einer anderen Stelle, so muss
;				der Page Befehl entsprechend angepasst werden!!!
;
;				Zudem muss vor dieser Routine das Modul SPI_UNIVERSAL.INC geladen werden
;				
;				Bsp:	
;					include "SPI_UNIVERSAL.inc"			;Laden der Subroutinen fr SPI INTERFACE
;					include "MCP2515.inc"				;Laden der Funktionen fr CAN Bus (MCP2515)
;
;------------------------------------------------------------------------------------------------------------------------------------------------------

;======================================================================================================================================================
; Anzupassen an jeweilige Schaltung !!!

;---------------Steuerleitungen des SX fr MCP2515-----------
MCP2515CS_SX			equ	RA.7			;CS vom 2515
MCP2515RESET_SX			equ	RD.2			;Reset-Pin des MCP 2515
MCP2515INT_SX			equ	RB.0

;Konstante fr CAN Kommunikation
;Empfangsadresse
BaseAdrCanRXH			equ	$34			;CANH Adresse der Platine =>$34*8
BaseAdrCanRXL			equ	$00			;CANL Adresse der Platine (nur die obersten 3Bit sind relevant)

								; $00 fr X-/C-Achse 	
								; $40 fr Z-/(C o. B)- Achse				
								; $80 fr A- gegf. Y-Achse				
								; $C0 fr X- /Y-Achse FTIR Positioniertisch		
;Sendadresse
BaseAdrCanTXH			equ	$34
BaseAdrCanTXL			equ	$20

;Kennung im CAN Bus
CanNodeKennung			equ	$12			
;------------------------------------------------------------


;======================================================================================================================================================
;***********************************AB HIER MGLICHST KEINE NDERUNGEN MEHR VORNEHMEN !!!**************************************************************
;Hilfsvariable
HILFSREG			equ 	$0F

;-------------------------------- Variablen - Definitionen ---------------------------
MCP2515_CountReg		equ	$80			;Zhlregister fr sendbyte und getbyte
MCP2515_InstructReg		equ	$81			;Register fr die SPI - Befehle des MCP2515
MCP2515_MaskReg			equ	$82			;Adresse des Mask Bytes fr den BitModify - Befehl/ die letzten 3 Bit fr RequestToSend_2515 (B'00000nnn')
MCP2515_CanFlags		equ	$83

MCP2515_AdressReg		equ	$84			;Register in dem die Adresse des MCP2515 - Registers steht, auf die ein spi2515Befehl angewendet werden soll
MCP2515_Data0			equ	$85			;Zeiger auf das erste Datenbyte fr den MCP2515
MCP2515_Data1			equ	$86			;Zeiger auf das zweite Datenbyte fr den MCP2515
;-------------------------------------------------------------------------------------

;-------------Definition von Buffern zum Senden und Empfangen von MSG-----------------
;Definition des MSG Buffers zum Senden in der SX Registerbank $9X
CANS_MSG_ADDRH			equ	$90			;Adresse CAN High
CANS_MSG_ADDRL			equ	$91			;Adresse CAN Low
CANS_MSG_DBYTE			equ	$92			;Anzahl Datenbytes
CANS_MSG_DATA0			equ	$93			;Daten Byte 4
CANS_MSG_DATA1			equ	$94			;Daten Byte 5
CANS_MSG_DATA2			equ	$95			;Daten Byte 6
CANS_MSG_DATA3			equ	$96			;Daten Byte 7
CANS_MSG_DATA4			equ	$97			;Daten Byte 8
CANS_MSG_DATA5			equ	$98			;Daten Byte 9
CANS_MSG_DATA6			equ	$99			;Daten Byte 10
CANS_MSG_DATA7			equ	$9A			;Daten Byte 11
CAN_SEND_STAT			equ	$9B			;Statusregister, zeigt Zustand der zu Sendenden Daten an 
CAN_SEND_EXT			equ	$9C			;Register fr extern Identifier


;Definition des MSG Buffers zum Empfangen in der SX Registerbank $AX
CANE_MSG_ADDRH			equ	$A0			;Adresse CAN High
CANE_MSG_ADDRL			equ	$A1			;Adresse CAN Low
CANE_MSG_DBYTE			equ	$A2			;Anzahl Datenbytes
CANE_MSG_DATA0			equ	$A3			;Daten Byte 4
CANE_MSG_DATA1			equ	$A4			;Daten Byte 5
CANE_MSG_DATA2			equ	$A5			;Daten Byte 6
CANE_MSG_DATA3			equ	$A6			;Daten Byte 7
CANE_MSG_DATA4			equ	$A7			;Daten Byte 8
CANE_MSG_DATA5			equ	$A8			;Daten Byte 9
CANE_MSG_DATA6			equ	$A9			;Daten Byte 10
CANE_MSG_DATA7			equ	$AA			;Daten Byte 11
CAN_EMPF_STAT			equ	$AB			;Statusregister, zeigt Zustand der Empfangenen Daten an 
CAN_EMPF_EXT			equ	$AC			;Register fr extern Identifier
;--------------------------------------------------------------------------------------


;============================= Definitionen von Konstanten und Register Adressen des MCP2515/MCP2515 =================================
;---------SPI - Befehlssatz des MCP2515  -  Konstanten----------
spi2515Befehl_Reset		equ	%11000000
spi2515Befehl_Read		equ	%00000011
spi2515Befehl_Write		equ	%00000010
spi2515Befehl_RTS		equ	%10000000		;ACHTUNG! RTS - Befehl (Request To Send) sieht so aus: "10000nnn". Die n's bedeuten 
								;(von links nach rechts):RTS fr TXB2, TXB1, TXB0. (TXB = transmit buffer)
spi2515Befehl_ReadStatus	equ	%10100000
spi2515Befehl_RxStatus		equ	%10110000
spi2515Befehl_BitModify		equ	%00000101
;---------------------------------------------------------------


;---------------Adressen der Register des MCP2515---------------
CANSTAT				equ	%00001110		;fr weitere Infos: MCP2515 Datenblatt S.53
CANCTRL				equ	%00001111

CNF1				equ	$2A
CNF2				equ	$29
CNF3				equ	$28

CANINTE				equ	$2B
CANINTF				equ	$2C

;Adressen der Filter - Masken - Register fr Receive Buffer 0 - alles fr Standard - Identifier
RXM0SIDH			equ	$20			;Adresse des Hi- und des Lo- Bytes der Acceptance Filter Mask 
RXM0SIDL			equ	$21			;fr Receive Buffer 0

;Adressen der Filter - Register fr Receive Buffer 0 - alles fr Standard - Identifier
RXF0SIDH			equ	$00			;Adressen von Acceptance Filter 0 - 1 (fr RXB0)
RXF0SIDL			equ	$01
RXF1SIDH			equ	$04
RXF1SIDL			equ	$05

;Adressen der Filter - Masken - Register fr Receive Buffer 1- alles fr Standard - Identifier
RXM1SIDH			equ	$24			;Adresse des Hi- und des Lo- Bytes der Acceptance Filter Mask
RXM1SIDL			equ	$25			;fr Receive Buffer 1

;Adressen der Filter - Register fr Receive Buffer 1 - alles fr Standard - Identifier
RXF2SIDH			equ	$08			;Adressen von Acceptance Filter 2 - 5 (fr RXB1)
RXF2SIDL			equ	$09
RXF3SIDH			equ	$10
RXF3SIDL			equ	$11
RXF4SIDH			equ	$14
RXF4SIDL			equ	$15
RXF5SIDH			equ	$18
RXF5SIDL			equ	$19

;wichtige Adressen fr Receive Buffer 0 (RXB0)
RXB0CTRL			equ	$60			;Kontroll - Register von RXB0
RXB0SIDH			equ	$61			;Adresse des Hi - Bytes vom Standard - ID RXB0  --  Lo - Byte direkt dahinter
RXB0SIDL			equ	$62			;Adresse des Hi - Bytes vom Standard - ID RXB0  --  Lo - Byte direkt dahinter
RXB0DLC				equ	$65			;hier befindet sich der Daten - Lngen - Code (Bits 0 bis 3)
RXB0D0				equ	$66			;Adresse des ersten Daten - Byte's

;wichtige Adressen fr Receive Buffer 1 (RXB1)
RXB1CTRL			equ	$70			;Kontroll - Register von RXB1
RXB1SIDH			equ	$71			;Adresse des Hi - Bytes vom Standard - ID RXB1  --  Lo - Byte direkt dahinter
RXB1SIDL			equ	$72			;Adresse des Hi - Bytes vom Standard - ID RXB1  --  Lo - Byte direkt dahinter
RXB1DLC				equ	$75			;hier befindet sich der Daten - Lngen - Code (Bits 0 bis 3)
RXB1D0				equ	$76			;Adresse des ersten Daten - Byte's
RXB1D1				equ	$77
RXB1D2				equ	$78
RXB1D3				equ	$79
RXB1D4				equ	$7A
RXB1D5				equ	$7B
RXB1D6				equ	$7C
RXB1D7				equ	$7D

;wichtige Adressen fr Transmit Buffer 0 (TXB0)
TXB0CTRL			equ	$30			;Kontroll - Register von TXB0
TXB0SIDH			equ	$31			;Adresse des Hi - Bytes vom Standard - ID TXB0  --  Lo - Byte direkt dahinter
TXB0SIDL			equ	$32			;Adresse des Hi - Bytes vom Standard - ID TXB0  --  Lo - Byte direkt dahinter
TXB0DLC				equ	$35			;hier befindet sich der Daten - Lngen - Code (Bits 0 bis 3)
TXB0D0				equ	$36			;Adresse des ersten Daten - Bytes
TXB0D1				equ	$37			;Adresse des ersten Daten - Bytes
TXB0D2				equ	$38			;Adresse des ersten Daten - Bytes
TXB0D3				equ	$39			;Adresse des ersten Daten - Bytes
TXB0D4				equ	$3A			;Adresse des ersten Daten - Bytes
TXB0D5				equ	$3B			;Adresse des ersten Daten - Bytes
TXB0D6				equ	$3C			;Adresse des ersten Daten - Bytes
TXB0D7				equ	$3D			;Adresse des ersten Daten - Bytes

point_TXBOSIDH			equ	%01000000		;Pointer fr Load TX Buffer TXBOSIDH
point_TXBOD0			equ	%01000001		;Pointer fr Load TX Buffer TXB0D0
point_TXB1SIDH			equ	%01000010		;Pointer fr Load TX Buffer TXB1SIDH
point_TXB1D0			equ	%01000011		;Pointer fr Load TX Buffer TXB1D0
point_TXB2SIDH			equ	%01000100		;Pointer fr Load TX Buffer TXB2SIDH
point_TXB2D0			equ	%01000101		;Pointer fr Load TX Buffer TXB2D0

point_RXBOSIDH			equ	%10010000		;Pointer fr READ RX Buffer RXBOSIDH
point_RXBOD0			equ	%10010010		;Pointer fr READ RX Buffer RXB0D0
point_RXB1SIDH			equ	%10010100		;Pointer fr READ RX Buffer RXB1SIDH
point_RXB1D0			equ	%10010110		;Pointer fr READ RX Buffer RXB1D0


;wichtige Adressen fr Transmit Buffer 1 (TXB1)
TXB1CTRL			equ	$40			;Kontroll - Register von TXB1
TXB1SIDH			equ	$41			;Adresse des Hi - Bytes vom Standard - ID TXB1  --  Lo - Byte direkt dahinter
TXB1DLC				equ	$45			;hier befindet sich der Daten - Lngen - Code (Bits 0 bis 3)
TXB1D0				equ	$46			;Adresse des ersten Daten - Bytes

;wichtige Adressen fr Transmit Buffer 2 (TXB2)
TXB2CTRL			equ	$50			;Kontroll - Register von TXB2
TXB2SIDH			equ	$51			;Adresse des Hi - Bytes vom Standard - ID TXB2  --  Lo - Byte direkt dahinter
TXB2DLC				equ	$55			;hier befindet sich der Daten - Lngen - Code (Bits 0 bis 3)
TXB2D0				equ	$56			;Adresse des ersten Daten - Bytes

preset_CNF1			equ	$07			;Physical Layer Configuration (= Einstellungen in der CNF - Registern des MCP2515) 
preset_CNF2			equ	$90			;aus dem CAN-NODE - Programm fr den PIC entnommen
preset_CNF3			equ	$02
preset_CANINTE			equ	$00			;Einstellung des Interrupt Enable Registers
preset_RXB0CTRL			equ	%00100100		;Masken - Einstellungen in RXB0CTRL - Maske: 01100100 
preset_RXB1CTRL			equ	%00100000		;Masken - Einstellungen in RXB1CTRL - Maske: 01100000 

;fr Receive Buffer 0 - alles fr Standard - Identifier   --  Voreinstellungen
preset_RXM0SIDH			equ	$FF		
preset_RXM0SIDL			equ	$00		

preset_RXF0SIDH			equ	$00			;blo vorlufig, wird spter mit der Adresse vom DIP geladen
preset_RXF0SIDL			equ	$00
preset_RXF1SIDH			equ	$00			;blo vorlufig, wird spter mit der Adresse vom DIP geladen
preset_RXF1SIDL			equ	$00

;fr Receive Buffer 1- alles fr Standard - Identifier  --  Voreinstellungen
preset_RXM1SIDH			equ	$FF			
preset_RXM1SIDL			equ	%11100000

preset_RXF2SIDH			equ	$00			;blo vorlufig, wird spter mit der Adresse vom DIP geladen			
preset_RXF2SIDL			equ	%11100000
preset_RXF3SIDH			equ	$00			;blo vorlufig, wird spter mit der Adresse vom DIP geladen
preset_RXF3SIDL			equ	%11100000
preset_RXF4SIDH			equ	$00			;blo vorlufig, wird spter mit der Adresse vom DIP geladen
preset_RXF4SIDL			equ	%11100000
preset_RXF5SIDH			equ	$00			;blo vorlufig, wird spter mit der Adresse vom DIP geladen
preset_RXF5SIDL			equ	%11100000

;Standardadresse fr den SPI Buffers
preset_StandardAdr		equ	$16			;Standard SX: $20 im MCP2515_Data0 jederzeit nderbar
;---------------------------------------------------------------
;=====================================================================================================================================================

;>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> Subroutinen zur Kommunikation >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

;======= Macros zur definierten Zeitverzgerung,ChipSelect und Clock - Steuerung fr Serial ================

;------Macro whlt den MCP an-----
MCP2515_Select	macro
		clrb SCK_SX		;Clock LOW setzen
		nop
		nop					
		nop
		clrb MCP2515CS_SX	;Chip Select aktivieren
		nop
		nop
endm
;---------------------------------


;------Macro whlt den MCP ab-----
MCP2515_UnSelect	macro
		clrb SCK_SX		;Clock LOW setzen
		nop
		nop	
		nop
		nop				
		setb MCP2515CS_SX	;Chip Select deaktivieren
endm
;---------------------------------


;-Macro aktiviert RESET des MCP-
MCP2515RESET_AK		macro
		;Ausgang Y4
		nop
		nop
		clrb MCP2515RESET_SX	;Reset MCP2515 (LOW)
		nop
endm
;---------------------------------


;-Macro deaktiviert RESET des MCP-
MCP2515RESET_DK		macro
		setb MCP2515RESET_SX	;Reset deaktivieren MCP2515 (HIGH)
endm
;---------------------------------


;-Macro setzt _counter, adress_reg und den Buffer_pointer-
SPI_Write	macro 2					;drei Parameter 1:Adress_Reg, 2:Byte_Counter, 3: Buffer_Pointer
		mov FSR,#MCP2515_AdressReg		;MCP2515_AdressReg laden
		mov INDF,#\1	
		INC FSR					;MCP2515_Data0	laden
		mov INDF,#\2				        ;zu bertrageneder Wert ist direkt im Aufruf
endm
;-------------------------------------------------------


;-Macro setzt _counter und adress_reg-
SPI_READ	macro 1					;zwei Parameter
		mov FSR,#MCP2515_AdressReg		;MCP2515_AdressReg laden
		mov INDF,#\1
endm
;-------------------------------------------------------


;-Macro setzt _adress_reg, _mask_reg und Buffer_pointer
SPI_BitMod	macro 3
		mov FSR,#MCP2515_AdressReg
		mov INDF,#\1
		mov FSR,#MCP2515_MaskReg
		mov INDF,#\2
		mov FSR,#MCP2515_Data0
		mov INDF,#\3
endm
;-------------------------------------------------------
;========================================================================================================================================

;Speicherbereich fr Definition von Sprungmarken fr MCP Routinen
org $A00		

LLOAD_TX_BUFFER		jmp LOAD_TX_BUFFER
LREAD_RX_BUFFER		jmp READ_RX_BUFFER
LREAD_ST_INSTR		jmp READ_ST_INSTR
LRX_ST_INSTR		jmp RX_ST_INSTR
;============================= Kommunikation mit dem MCP2515 ============================================================================

;-Subroutine zum Schreiben in MCP2515-
WRITE_INSTRUCTION	
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Write - Befehls ins Befehlsregister laden
		mov INDF,#spi2515Befehl_Write				;fhrt den SPI Write Befehl des MCP2515 aus,
		page $C00
			call sendbyte						;Write - Befehl senden
		mov FSR,#MCP2515_AdressReg				;Adresse des Registers mit der MCP2515 - Register - Adresse laden
			call sendbyte						;Adresse senden
		mov FSR,#MCP2515_Data0				;die Daten ab diesem Register soll in das MCP2515 - Register		
			call sendbyte						;Byte senden
		page $A00
		MCP2515_UnSelect			;MCP2515 abwhlen
ret
;-------------------------------------

;-Subroutine zum Lesen aus MCP2515-
READ_INSTRUCTION	
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#spi2515Befehl_Read				;fhrt den SPI Read Befehl des MCP2515 aus,		
		page $C00
		    call sendbyte						;Read - Befehl senden
		mov FSR,#MCP2515_AdressReg				;Adresse des Registers mit der MCP2515 - Register - Adresse laden
		    call sendbyte						;Adresse senden
		mov FSR,#MCP2515_Data0					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call getbyte						;Byte vom MCP2515 empfangen
		page $A00
		MCP2515_UnSelect			;MCP2515 abwhlen
ret
;-------------------------------------

;-----Subroutine zum ausfhren der BitModofy Routine-----
BIT_MODIFY_INSTRUCTION	
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Write - Befehls ins Befehlsregister laden
		mov INDF,#spi2515Befehl_BitModify			;fhrt den SPI BitModify Befehl des MCP2515 aus,
		page $C00
			call sendbyte						;BitModify - Befehl senden
		mov FSR,#MCP2515_AdressReg				;Adresse des Registers mit der MCP2515 - Register - Adresse laden
			call sendbyte						;Adresse senden
		mov FSR,#MCP2515_MaskReg				;Mask - Byte fr die BitModify - Operation laden
			call sendbyte						;Mask - Byte senden
		mov FSR,#MCP2515_Data0			;Daten - Byte fr die BitModify - Operation laden
			call sendbyte					;Daten - Byte senden
		page $A00
		MCP2515_UnSelect
ret
;-------------------------------------

;----------------- Setzen der CAN ID -------------------
ICANSADR
		mov FSR,#CANS_MSG_ADDRH					;Adresse CAN High
		mov INDF,#BaseAdrCanTXH
	
		mov FSR,#CANS_MSG_ADDRL					;Adresse CAN Low
		mov INDF,#BaseAdrCanTXL
ret
;-------------------------------------------------------

;====================================== Init_2515  --  Initialisierungs - MCP2515 ===========================================================
;-- Einstellungen: siehe Voreinstellungen fr den MCP2515 (weiter oben) --
Init_2515	
		;------Implementation der Befehle des MCP2515 - SPI - Interfaces----
		;-Reset des MCP2515 ber Reset - Pin-
		MCP2515RESET_AK
		mov FSR,#COUNTER0					;mindestens 2us warten
		mov INDF,#$90	

		page $E00						
		     call TimeEater1
		page $A00

		MCP2515RESET_DK						;Reset deaktivieren
		;-------------------------------------------------------
	
		;------------------PhysicalLayerConfiguration--------------------
		;Set physical layer configuration (Config Mode)
		SPI_WRITE CANCTRL,$84					;Adresse laden (%00001111)
									;   100  =>> Config Mode,  => Setze b100 von CANCTRL (vorher b110)
		mov FSR,#MCP2515_MaskReg				;Maske laden: die 3 niedrigesten Bits sollen gendert werden: REQOP<2:0>
		mov INDF,#%11100111					
		
		;-SPI BitModify Befehl-
		     call BIT_MODIFY_INSTRUCTION
		;-------------------------------------------------------
	
		SPI_Write CNF1,$07  					; set BRP to div by 8
		     call WRITE_INSTRUCTION

		SPI_Write CNF2,$90
		     call WRITE_INSTRUCTION

		SPI_Write CNF3,$02
		     call WRITE_INSTRUCTION
		;--------------------------------------------------------------


		;----------------i2515_Filter_Masken setzen--------------------
		;--Configure Receive buffer 0 Mask and Filters--
		;-Receive buffer 0 will not be used-
		SPI_Write RXM0SIDH,$FF
		     call WRITE_INSTRUCTION

		SPI_Write RXM0SIDL,$FF
		     call WRITE_INSTRUCTION

		SPI_Write RXF0SIDH,$FF
		     call WRITE_INSTRUCTION

		SPI_Write RXF0SIDL,$FF
		     call WRITE_INSTRUCTION

		SPI_Write RXF1SIDH,$FF
		     call WRITE_INSTRUCTION

		SPI_Write RXF1SIDL,$FF
		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;--Configure Receive Buffer 1 Mask and Filters--
		SPI_Write RXM1SIDH,$FF
		     call WRITE_INSTRUCTION

		SPI_Write RXM1SIDL,$E0
		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;--Initialize Filter 2 to match x0 bBaseRecID---
		SPI_Write RXF2SIDH,BaseAdrCanRXH
		     call WRITE_INSTRUCTION
		
		SPI_Write RXF2SIDL,BaseAdrCanRXL ; Make sure EXIDE bit (bit 3)
		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;is set correctly in filter
		;--Initialize Filter 3 to match x1 bBaseRecID+1-
		SPI_Write RXF3SIDH,BaseAdrCanRXH
		     call WRITE_INSTRUCTION
			
		SPI_Write RXF3SIDL,BaseAdrCanRXL
		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;--Initialize Filter 4 to match x2 bBaseRecID+2-
		SPI_Write RXF4SIDH,BaseAdrCanRXH	
		     call WRITE_INSTRUCTION

		SPI_Write RXF4SIDL,BaseAdrCanRXL
		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;--Initialize Filter 5 to match x3 bBaseRecID+3-
		SPI_Write RXF5SIDH,BaseAdrCanRXH
		     call WRITE_INSTRUCTION

		SPI_Write RXF5SIDL,BaseAdrCanRXL
		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;---------------i2515_InterruptsAus-------------
		SPI_Write CANINTE,$00
 		     call WRITE_INSTRUCTION
		;-----------------------------------------------

		;-----------Wechsel in NormalMode_2515----------
		SPI_WRITE CANCTRL,$00					;Adresse laden (%00001111)
									;   weil REQOP<2:0> = 0  =>> Normal Mode,  => Setze b110 von CANCTRL
		mov FSR,#MCP2515_MaskReg				
		mov INDF,#%11100000					;Maske laden: die 3 hchsten Bits sollen gendert werden: REQOP<2:0>				
		;=>lschen der Bits 7,6,5 von CANCTRL
		;-SPI BitModify Befehl-
		     call BIT_MODIFY_INSTRUCTION
		;-------------------------------------------------------

Wait_Mode	
		SPI_READ CANSTAT					;Adresse des CANSTAT - Registers laden	
		     call READ_INSTRUCTION
		
		mov FSR,#MCP2515_Data0
		and INDF,#%11100000    			;Maskieren
		   JNZ Wait_Mode	               	;wenn Ungleich Null Sprung zu Label
ret
;------------------------------------------------------------------
;==============================================================================================================================================================


;-----Subroutine zum Ausfhren der LOAD TX BUFFER Routine-----
LOAD_TX_BUFFER						;fhrt den SPI LOAD TX BUFFER Befehl des MCP2515 aus 
		;Nachricht zusammenbasteln
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#point_TXBOSIDH					;Laden BUFFER TXB0SIDH
		page $C00
		    call sendbyte
		mov FSR,#CANS_MSG_ADDRH					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call sendbyte					;Daten senden
		mov FSR,#CANS_MSG_ADDRL
		and INDF,#%11100000
    		    call sendbyte
		mov FSR,#CAN_SEND_EXT
		clr INDF						;Extended Identifier H				;
    		    call sendbyte
		clr INDF
    		    call sendbyte
		mov FSR,#CANS_MSG_DBYTE	
    		    call sendbyte					;Daten senden
		page $A00
		MCP2515_UnSelect			;MCP2515 abwhlen

		
		mov HILFSREG,#CANS_MSG_DATA0				;Laden CANS_MSG_Data0 

		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#point_TXBOD0
		page $C00
		    call sendbyte						;Befehl senden
		page $A00

REPEAT_LOADTX
		mov FSR,HILFSREG					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		page $C00
		    call sendbyte						;Byte vom MCP2515 empfangen
		page $A00

		INC HILFSREG

		mov FSR,#CANS_MSG_DBYTE	
		DEC INDF						;Hilfsregister um 1 erhhen
		sz
		    jmp REPEAT_LOADTX			;Sprung zurck an Anfang der Schleife
		MCP2515_UnSelect			;MCP2515 abwhlen
ret
;-------------------------------------


;-----Subroutine zum Ausfhren der READ RX BUFFER Routine-----
READ_RX_BUFFER						;fhrt den SPI LOAD TX BUFFER Befehl des MCP2515 aus 
		;Nachricht zusammenbasteln
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#point_RXB1SIDH					;Laden BUFFER TXB0SIDH
		page $C00
		    call sendbyte
		mov FSR,#CANE_MSG_ADDRH					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call getbyte					;Daten senden
		mov FSR,#CANE_MSG_ADDRL
    		    call getbyte
		mov FSR,#CAN_EMPF_EXT					;Extended Identifier H				;
    		    call getbyte
		clr INDF
    		    call getbyte
		mov FSR,#CANE_MSG_DBYTE	
    		    call getbyte					;Daten senden
		page $A00
		MCP2515_UnSelect			;MCP2515 abwhlen

		mov HilfSpeich0,INDF					;Anzahl der bertragenen Bytes
		
		mov HILFSREG,#CANE_MSG_DATA0				;Laden CANS_MSG_Data0 

		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#point_RXB1D0
		page $C00
		    call sendbyte						;Befehl senden
		page $A00

REPEAT_READRX
		mov FSR,HILFSREG					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		page $C00
		    call getbyte						;Byte vom MCP2515 empfangen
		page $A00

		INC HILFSREG

		DEC HilfSpeich0						;Hilfsregister um 1 erhhen
		sz
		    jmp REPEAT_READRX			;Sprung zurck an Anfang der Schleife
		MCP2515_UnSelect			;MCP2515 abwhlen
ret
;-------------------------------------


;-----Subroutine zum Ausfhren der READ STATUS INSTRUCTION Routine-----
READ_ST_INSTR
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#spi2515Befehl_ReadStatus
		page $C00
		    call sendbyte						;Befehl senden
		mov FSR,#MCP2515_Data0					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call getbyte						;Byte vom MCP2515 empfangen
		mov FSR,#MCP2515_Data1					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call getbyte						;Byte vom MCP2515 empfangen
		page $A00
		MCP2515_UnSelect			;MCP2515 abwhlen
ret
;-------------------------------------


;-----Subroutine zum Ausfhren der READ RX STATUS INSTRUCTION Routine-----
RX_ST_INSTR
		MCP2515_Select				;MCP2515 auswhlen
		mov FSR,#MCP2515_InstructReg				;Code des Read - Befehls ins Befehlsregister laden
		mov INDF,#spi2515Befehl_RxStatus
		page $C00
		    call sendbyte						;Befehl senden
		mov FSR,#MCP2515_Data0					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call getbyte						;Byte vom MCP2515 empfangen
		mov FSR,#MCP2515_Data1					;hier sollen die Daten aus dem MCP in den SX gespeichert werden
		    call getbyte						;Byte vom MCP2515 empfangen
		page $A00
		MCP2515_UnSelect			;MCP2515 abwhlen
ret
;-------------------------------------

;Speicherbereich fr andere Routinen des MCP2515
org $C00
;===================================== Routinen zum Senden und Empfangen ber CAN Bus ==================================================
;------------- Subroutine zum Senden der der MSG an den MCP2515 ---------------------
SendCanMsg

WaitTXMsg_Ready	
		SPI_Read TXB0CTRL
		page $A00
		     call READ_INSTRUCTION					;Warten bis alle Nachtrichten versand sind
		page $C00

		mov FSR,#MCP2515_Data0
		and INDF,#$08			
		    JNZ WaitTXMsg_Ready				;Warten bis Null

		page $A00
		     call LLOAD_TX_BUFFER
		page $C00
SendMSGEND
		;-------REMOTE TO SEND-------
		mov FSR,#MCP2515_InstructReg			;Code des Write - Befehls ins Befehlsregister laden
		mov INDF,#spi2515Befehl_RTS				;fhrt den SPI Write Befehl des MCP2515 aus
		or  INDF,#$01
	
		MCP2515_Select						;MCP2515 auswhlen
		     call sendbyte					;Write - Befehl senden
		MCP2515_UnSelect					;MCP2515 abwhlen
		;----------------------------

		;----Zurcksetzen der CAN Flags--
		mov FSR,#MCP2515_CanFlags					
		mov INDF,#$00
ret
;------------------------------------------------------------------------------------------------------------------------------


;---------------------- Subroutine zum Empfang einer Nachricht von MCP2515/MCP2515 -------------------------
CheckCanMsg	
		;-------------Prfen ob CAN Flag gesetzt-------------
		mov FSR,#MCP2515_CanFlags
		snb INDF.0
		    ret		;wenn nicht Routine beenden
		;----------------------------------------------------


		;-------berprfen ob neue Nachricht empfangen-------	
		page $A00	
		     call LREAD_ST_INSTR
		page $C00

		mov FSR,#MCP2515_Data0
		and INDF,#%00000010		;Msg in RXB1 Buffer
		snz				
		   ret				;wenn gleich Null beenden der Routine

		;-Flags setzen-
		mov FSR,#MCP2515_CanFlags
		setb INDF.0
		;----------------------------------------------------
		;-Lesen der CAN Nachricht
		page $A00
		     call LREAD_RX_BUFFER
		page $C00
		
		;-------------------------------------------
				
		;--------- SPI_BitMod CANINTF,$02,0 --------
endDataRead	
		SPI_BitMod CANINTF,$02,0
		
		page $A00
	             call BIT_MODIFY_INSTRUCTION
		page $C00	
		;----------------------------------------------------
ret

;--------------------------------------------------------------------
